import 'dart:async';
import 'dart:io';

import 'package:eyflutter_core/device/android_device_info.dart';
import 'package:eyflutter_core/device/device_channels.dart';
import 'package:eyflutter_core/device/device_info.dart';
import 'package:eyflutter_core/device/ios_device_info.dart';
import 'package:eyflutter_core/kit/utils/dynamic_utils.dart';
import 'package:eyflutter_core/kit/utils/json_utils.dart';
import 'package:eyflutter_core/kit/utils/string/json_extension.dart';
import 'package:eyflutter_core/kit/utils/string/string_extension.dart';
import 'package:eyflutter_core/mq/cloud_channel_manager.dart';
import 'package:eyflutter_core/storage/mmkv_utils.dart';

class DeviceUtils {
  factory DeviceUtils() => _getInstance();

  static DeviceUtils get instance => _getInstance();
  static DeviceUtils? _instance;

  static DeviceUtils _getInstance() {
    return _instance ??= new DeviceUtils._internal();
  }

  DeviceUtils._internal();

  String _deviceMethodName = "1c94d6dafa7e7903";
  String _deviceCacheKey = "ec84902b39acefa2";
  Completer<AndroidDeviceInfo> _androidDeviceCompleter = Completer<AndroidDeviceInfo>();
  Completer<IosDeviceInfo> _iosDeviceCompleter = Completer<IosDeviceInfo>();

  /// 获取设备信息
  Future<DeviceInfo> get deviceInfo => _getDeviceInfo();

  Future<DeviceInfo> _getDeviceInfo() async {
    if (Platform.isAndroid) {
      var deviceInfo = await _getAndroidDeviceInfo();
      return _toDeviceInfoWithAndroid(deviceInfo);
    } else if (Platform.isIOS) {
      var deviceInfo = await _getIOSDeviceInfo();
      return _toDeviceInfoWithIos(deviceInfo);
    } else {
      return Future.value(DeviceInfo());
    }
  }

  Future<DeviceInfo> _toDeviceInfoWithIos(IosDeviceInfo iosDeviceInfo) {
    var deviceInfo = DeviceInfo();
    deviceInfo.deviceId = iosDeviceInfo.deviceId;
    deviceInfo.model = iosDeviceInfo.model;
    deviceInfo.brand = iosDeviceInfo.brand;
    deviceInfo.density = iosDeviceInfo.density;
    deviceInfo.display = iosDeviceInfo.resolution;
    deviceInfo.deviceType = 2;
    deviceInfo.sys = "${iosDeviceInfo.systemName} ${iosDeviceInfo.sysVersion}";
    deviceInfo.root = iosDeviceInfo.isRoot.isTrue;
    deviceInfo.simulator = iosDeviceInfo.isSimulator.isTrue;
    return Future.value(deviceInfo);
  }

  Future<DeviceInfo> _toDeviceInfoWithAndroid(AndroidDeviceInfo androidDeviceInfo) {
    var deviceInfo = DeviceInfo();
    deviceInfo.deviceId = androidDeviceInfo.deviceId;
    deviceInfo.simulator = androidDeviceInfo.isSimulator.isTrue;
    deviceInfo.root = androidDeviceInfo.isRoot.isTrue;
    deviceInfo.display = androidDeviceInfo.resolution;
    deviceInfo.hardware = androidDeviceInfo.hardware;
    deviceInfo.deviceType = 1;
    deviceInfo.density = androidDeviceInfo.density;
    deviceInfo.board = androidDeviceInfo.board;
    deviceInfo.brand = androidDeviceInfo.brand;
    deviceInfo.abis = androidDeviceInfo.abis;
    deviceInfo.model = androidDeviceInfo.model;
    deviceInfo.host = androidDeviceInfo.host;
    deviceInfo.manufacturer = androidDeviceInfo.manufacturer;
    deviceInfo.sys = "${androidDeviceInfo.deviceName} ${androidDeviceInfo.sysVersion}";
    Map<String, String> extensionMap = {
      "androidId": androidDeviceInfo.androidId,
      "cpuCoreNum": androidDeviceInfo.cpuCoreNum,
      "kernelVersion": androidDeviceInfo.kernelVersion,
      "mac": androidDeviceInfo.mac,
      "runMemory": androidDeviceInfo.runMemory,
      "storageSize": androidDeviceInfo.storageSize
    };
    deviceInfo.extension = JsonUtils.toJson(extensionMap);
    return Future.value(deviceInfo);
  }

  Future<AndroidDeviceInfo> _getAndroidDeviceInfo() async {
    if (_androidDeviceCompleter.isCompleted) {
      return _androidDeviceCompleter.future;
    }
    var cacheDevice = await _getAndroidCacheDevice();
    if (cacheDevice != null) {
      return cacheDevice;
    }
    String response = await CloudChannelManager.instance.send(_deviceMethodName);
    if (response.isEmptyJson) {
      return AndroidDeviceInfo();
    }
    var deviceMap = JsonUtils.fromJson(response);
    var deviceInfo = AndroidDeviceInfo.fromJson(deviceMap);
    deviceInfo.isRoot = "${await DeviceChannels.isRoot}";
    deviceInfo.isSimulator = "${await DeviceChannels.isSimulator}";
    var deviceJson = JsonUtils.toJson(deviceInfo);
    MmkvUtils.instance.putString(_deviceCacheKey, value: deviceJson, part: MMkvPart.normal);
    _androidDeviceCompleter.complete(deviceInfo);
    return _androidDeviceCompleter.future;
  }

  Future<AndroidDeviceInfo?> _getAndroidCacheDevice() async {
    var cacheDeviceData = await MmkvUtils.instance.getString(_deviceCacheKey, part: MMkvPart.normal);
    if (!cacheDeviceData.isEmptyJson) {
      var map = JsonUtils.fromJson(cacheDeviceData!);
      var device = AndroidDeviceInfo.fromJson(map);
      if (!device.deviceId.isEmptyString) {
        _androidDeviceCompleter.complete(device);
        return _androidDeviceCompleter.future;
      }
    }
    return null;
  }

  Future<IosDeviceInfo> _getIOSDeviceInfo() async {
    if (_iosDeviceCompleter.isCompleted) {
      return _iosDeviceCompleter.future;
    }
    var cacheDevice = await _getIOSCacheDevice();
    if (cacheDevice != null) {
      return cacheDevice;
    }
    String response = await CloudChannelManager.instance.send(_deviceMethodName);
    if (response.isEmptyJson) {
      return IosDeviceInfo();
    }
    var deviceMap = JsonUtils.fromJson(response);
    var deviceInfo = IosDeviceInfo.fromJson(deviceMap);
    deviceInfo.isRoot = "${await DeviceChannels.isRoot}";
    deviceInfo.isSimulator = "${await DeviceChannels.isSimulator}";
    var deviceJson = JsonUtils.toJson(deviceInfo);
    MmkvUtils.instance.putString(_deviceCacheKey, value: deviceJson, part: MMkvPart.normal);
    _iosDeviceCompleter.complete(deviceInfo);
    return _iosDeviceCompleter.future;
  }

  Future<IosDeviceInfo?> _getIOSCacheDevice() async {
    var cacheDeviceData = await MmkvUtils.instance.getString(_deviceCacheKey, part: MMkvPart.normal);
    if (!cacheDeviceData.isEmptyJson) {
      var map = JsonUtils.fromJson(cacheDeviceData!);
      var device = IosDeviceInfo.fromJson(map);
      if (!device.deviceId.isEmptyString) {
        _iosDeviceCompleter.complete(device);
        return _iosDeviceCompleter.future;
      }
    }
    return null;
  }
}
