import 'dart:async';

import 'package:eyflutter_core/dio/beans/retrofit.dart';
import 'package:eyflutter_core/dio/enums/interceptor_type.dart';
import 'package:eyflutter_core/dio/enums/method.dart';
import 'package:eyflutter_core/dio/event/dio_calls.dart';
import 'package:eyflutter_core/dio/event/dio_interceptor.dart';
import 'package:eyflutter_core/ey_flutter.dart';
import 'package:eyflutter_core/kit/timer/system.dart';
import 'package:eyflutter_core/kit/utils/dynamic_utils.dart';
import 'package:eyflutter_core/kit/utils/json_utils.dart';
import 'package:eyflutter_core/kit/utils/map/map_extension.dart';
import 'package:eyflutter_core/kit/utils/string/path_extension.dart';
import 'package:eyflutter_core/kit/utils/string/string_extension.dart';
import 'package:eyflutter_core/kit/utils/string/url_extension.dart';
import 'package:eyflutter_core/mq/cloud_channel_manager.dart';
import 'package:flutter/services.dart';
import 'package:uuid/uuid.dart';

typedef OnResponseSuccessCall = void Function(String requestId, Map<String, dynamic> dataMap);
typedef OnResponseErrorCall = void Function(String requestId, String message, Map<String, dynamic> dataMap);
typedef OnResponseStringSuccessCall = void Function(String requestId, String responseData);
typedef OnResponseStringErrorCall = void Function(String requestId, String message, String responseData);
typedef OnResponseCompleteCall = void Function(String requestId);
typedef OnProcessCall = void Function(double process);

class DioManager {
  factory DioManager() => _getInstance();

  static DioManager get instance => _getInstance();
  static DioManager? _instance;

  static DioManager _getInstance() {
    return _instance ??= new DioManager._internal();
  }

  String _requestMethodName = "e004dc5b1e384739";
  String _requestCompleteAction = "812454d85242c263";
  String _requestProcessAction = "f4f4cce92307e7f5";
  Map<String, dynamic> _interceptorMap = {};
  Map<String, OnResponseCompleteCall> _requestActMap = {};
  Map<String, Completer> _syncMap = {};
  Map<String, OnProcessCall> _processMap = {};

  DioManager._internal() {
    CloudChannelManager.instance.channel?.setMethodCallHandler((call) {
      if (call.method == _requestCompleteAction) {
        String requestId = call.arguments ?? "";
        _methodCall(call, requestId);
      } else if (call.method == _requestProcessAction) {
        _processCall(call);
      }
      return Future.value();
    });
  }

  void _processCall(MethodCall call) {
    if (call.arguments is! Map) {
      return;
    }
    var progress = call.arguments["progress"];
    if (progress is! double) {
      progress = 0;
    }
    var requestId = "${call.arguments["requestId"]}";
    var processCall = _processMap[requestId];
    if (processCall == null) {
      return;
    }
    processCall(progress);
  }

  void _methodCall(MethodCall call, String requestId) {
    if (call.arguments is! String) {
      return;
    }
    _finishCall(requestId);
    if (_requestActMap.containsKey(requestId)) {
      var actCall = _requestActMap[requestId];
      if (actCall != null) {
        actCall(requestId);
      }
      _requestActMap.remove(requestId);
    }
  }

  void _finishCall(String requestId) {
    if (_syncMap.containsKey(requestId)) {
      Completer? completer = _syncMap[requestId];
      if (!(completer?.isCompleted ?? false)) {
        completer?.complete(Future.value(""));
      }
      _syncMap.remove(requestId);
    }
  }

  /// 添加拦截器
  /// [type] 拦截器类型
  /// [interceptor] 子拦截器
  void addInterceptor({required InterceptorType type, required dynamic interceptor}) {
    _interceptorMap[type.name] = interceptor;
  }

  OnRequestInterceptor? _getConfigInterceptor() {
    var interceptorObj = _interceptorMap[InterceptorType.CONFIG.name];
    if (interceptorObj is! OnRequestInterceptor) {
      return null;
    }
    return interceptorObj;
  }

  OnRequestInfoInterceptor? _getInfoInterceptor() {
    var interceptorObj = _interceptorMap[InterceptorType.INFO.name];
    if (interceptorObj is! OnRequestInfoInterceptor) {
      return null;
    }
    return interceptorObj;
  }

  OnResponseInterceptor? _getResponseInterceptor() {
    var interceptorObj = _interceptorMap[InterceptorType.RESPONSE.name];
    if (interceptorObj is! OnResponseInterceptor) {
      return null;
    }
    return interceptorObj;
  }

  /// 获取dio配置拦截器
  OnRequestInterceptor? get configInterceptor => _getConfigInterceptor();

  String _getBaseUrl(Retrofit retrofit) {
    if (!(retrofit.api.isUrl)) {
      var addressConfig = EyFlutter.instance.addressConfig;
      var baseUrl = addressConfig?.baseUrl(hostType: retrofit.hostType) ?? "";
      return baseUrl.appendPath(retrofit.api);
    }
    return retrofit.api;
  }

  String _urlJoin(Retrofit retrofit, OnResponseStringCall? responseCall) {
    var url = _getBaseUrl(retrofit);
    var queIndex = url.indexOf("?");
    if (url.isEmptyString || queIndex == 0) {
      responseCall?.onComplete();
      return "";
    }
    if (url.indexOf("?") > 0) {
      url = "$url&time=${System.currentTimeMillis}";
    } else {
      url = "$url?time=${System.currentTimeMillis}";
    }
    return url;
  }

  Map<String, T> _mergeParams<T>(Map<String, T>? map, Map<String, T>? globalParams) {
    if (globalParams.isEmptyMap()) {
      return map ?? {};
    }
    var _map = map ?? {};
    globalParams!.forEach((key, value) {
      if (!_map.containsKey(key)) {
        _map[key] = value;
      }
    });
    return _map;
  }

  /// 网络请求实例
  /// [retrofit] 请求对象
  /// [params] 请求参数
  /// [responseCall] 响应回调
  void _request(
      {required Retrofit retrofit,
      Map<String, String>? headers,
      Map<String, dynamic>? params,
      required String requestId,
      OnResponseStringCall? responseCall}) async {
    //请求url地址
    var url = _urlJoin(retrofit, responseCall);
    //请求参数合并
    var infoInterceptor = _getInfoInterceptor();
    var _headers = _mergeParams<String>(headers, infoInterceptor?.onRequestHeaders());
    var _params = _mergeParams<dynamic>(params, infoInterceptor?.onRequestParams());
    //dio请求
    var resultData = await CloudChannelManager.instance.send<String>(_requestMethodName, arguments: {
      "method": retrofit.method.name,
      "requestId": requestId,
      "contentType": retrofit.contentType.name,
      "url": url,
      "headers": _headers,
      "data": _params,
    });
    if (resultData.isEmptyString) {
      responseCall?.onError("error", resultData ?? "");
      _requestActMap.remove(requestId);
    } else {
      responseCall?.onSuccess(resultData ?? "");
    }
  }

  /// 网络请求实例并解析成Map对象
  /// [retrofit] 请求对象
  /// [params] 请求参数
  /// [responseCall] 响应回调
  void _requestForMap(
      {required Retrofit retrofit,
      Map<String, String>? headers,
      Map<String, dynamic>? params,
      required String requestId,
      OnResponseCall? responseCall}) async {
    var interceptor = _getConfigInterceptor();
    var responseInterceptor = _getResponseInterceptor();
    this._request(
        retrofit: retrofit,
        requestId: requestId,
        headers: headers,
        params: params,
        responseCall: _InternalResponseStringCall(responseCall, interceptor, responseInterceptor));
  }

  /// 网络请求实例
  /// [retrofit] 请求对象
  /// [params] 请求参数
  /// [responseCall] 响应回调
  void requestString(
      {required Retrofit retrofit,
      Map<String, String>? headers,
      Map<String, dynamic>? params,
      OnResponseStringSuccessCall? successCall,
      OnResponseStringErrorCall? errorCall,
      OnResponseCompleteCall? completeCall}) async {
    String requestId = Uuid().v4().toString();
    if (completeCall != null) {
      _requestActMap[requestId] = completeCall;
    }
    var responseInterceptor = _getResponseInterceptor();
    _request(
        retrofit: retrofit,
        headers: headers,
        params: params,
        requestId: requestId,
        responseCall: _InternalStringCall(requestId, successCall, errorCall, completeCall, responseInterceptor));
  }

  Future<String> asyncRequestString(
      {required Retrofit retrofit,
      Map<String, String>? headers,
      Map<String, dynamic>? params,
      OnResponseStringErrorCall? errorCall,
      OnResponseCompleteCall? completeCall}) {
    String requestId = Uuid().v4().toString();
    if (completeCall != null) {
      _requestActMap[requestId] = completeCall;
    }
    var responseInterceptor = _getResponseInterceptor();
    _syncMap[requestId] = new Completer<String>();
    _request(
        retrofit: retrofit,
        headers: headers,
        params: params,
        requestId: requestId,
        responseCall: _InternalStringCall(requestId, (String requestId, String responseData) {
          var completer = _syncMap[requestId] as Completer<String>;
          completer.complete(Future.value(responseData));
        }, (String requestId, String message, String responseData) {
          var completer = _syncMap[requestId] as Completer<String>;
          completer.complete(Future.value(""));
          if (errorCall != null) {
            errorCall(requestId, message, responseData);
          }
        }, (String requestId) {
          _finishCall(requestId);
          if (completeCall != null) {
            completeCall(requestId);
          }
        }, responseInterceptor));
    var completer = _syncMap[requestId] as Completer<String>;
    return completer.future;
  }

  /// 网络请求实例
  /// [retrofit] 请求对象
  /// [params] 请求参数
  /// [responseCall] 响应回调
  void request(
      {required Retrofit retrofit,
      Map<String, String>? headers,
      Map<String, dynamic>? params,
      OnResponseSuccessCall? successCall,
      OnResponseErrorCall? errorCall,
      OnResponseCompleteCall? completeCall}) async {
    String requestId = Uuid().v4().toString();
    if (completeCall != null) {
      _requestActMap[requestId] = completeCall;
    }
    _requestForMap(
        retrofit: retrofit,
        headers: headers,
        params: params,
        requestId: requestId,
        responseCall: _InternalResponseCall(requestId, successCall, errorCall, completeCall));
  }

  Future<Map<String, dynamic>> asyncRequest(
      {required Retrofit retrofit,
      Map<String, String>? headers,
      Map<String, dynamic>? params,
      OnResponseErrorCall? errorCall,
      OnResponseCompleteCall? completeCall}) {
    String requestId = Uuid().v4().toString();
    if (completeCall != null) {
      _requestActMap[requestId] = completeCall;
    }
    _syncMap[requestId] = new Completer<Map<String, dynamic>>();
    _requestForMap(
        retrofit: retrofit,
        requestId: requestId,
        headers: headers,
        params: params,
        responseCall: _InternalResponseCall(requestId, (String requestId, Map<String, dynamic> dataMap) {
          var completer = _syncMap[requestId] as Completer<Map<String, dynamic>>;
          completer.complete(Future.value(dataMap));
        }, (String requestId, String message, Map<String, dynamic> dataMap) {
          var completer = _syncMap[requestId] as Completer<Map<String, dynamic>>;
          completer.complete(Future.value(dataMap));
          if (errorCall != null) {
            errorCall(requestId, message, dataMap);
          }
        }, (String requestId) {
          _finishCall(requestId);
          if (completeCall != null) {
            completeCall(requestId);
          }
        }));
    var completer = _syncMap[requestId] as Completer<Map<String, dynamic>>;
    return completer.future;
  }

  /// 文件下载
  /// [url] 文件网络地址
  /// [fileName] 下载至本地文件名
  /// [completeCall] 请求完成后回调
  void download({
    required String url,
    required Function(String path) successCall,
    OnProcessCall? processCall,
    OnResponseCompleteCall? completeCall,
  }) {
    String requestId = Uuid().v4().toString();
    if (processCall != null) {
      _processMap[requestId] = processCall;
    }
    if (completeCall != null) {
      _requestActMap[requestId] = completeCall;
    }
    Retrofit retrofit = Retrofit(api: url, method: Method.DOWNLOAD);
    this._request(
      retrofit: retrofit,
      requestId: requestId,
      responseCall: _DownloadResponseCall(completeCall, successCall, requestId, _processMap),
    );
  }
}

class _DownloadResponseCall with OnResponseStringCall {
  final OnResponseCompleteCall? completeCall;
  final Function(String path) successCall;
  final String requestId;
  bool isFinished = false;
  final Map<String, OnProcessCall> processCallMap;

  _DownloadResponseCall(this.completeCall, this.successCall, this.requestId, this.processCallMap);

  @override
  void onComplete() {
    if (!isFinished && completeCall != null) {
      isFinished = true;
      completeCall!(requestId);
      processCallMap.remove(requestId);
    }
  }

  @override
  void onError(String message, String responseData) {
    if (!isFinished && completeCall != null) {
      isFinished = true;
      completeCall!(requestId);
      processCallMap.remove(requestId);
    }
  }

  void _parsingData(String responseData) {
    var resultMap = JsonUtils.fromJson(responseData);
    String type = resultMap["type"];
    if (type.isEmptyString || type == "error") {
      this.onError("", "");
    } else if (type == "success") {
      String response = resultMap["data"];
      var map = JsonUtils.fromJson(response);
      successCall(map["path"]);
    }
    this.onComplete();
  }

  @override
  void onSuccess(String responseData) {
    _parsingData(responseData);
  }
}

class _InternalResponseCall with OnResponseCall {
  final OnResponseSuccessCall? successCall;
  final OnResponseErrorCall? errorCall;
  final OnResponseCompleteCall? completeCall;
  final String requestId;

  //是否已经回调结束
  bool _isFinishCall = false;

  _InternalResponseCall(this.requestId, this.successCall, this.errorCall, this.completeCall) {
    _isFinishCall = false;
  }

  @override
  void onComplete() {
    if (completeCall != null && !_isFinishCall) {
      _isFinishCall = true;
      completeCall!(requestId);
    }
  }

  @override
  void onError(String message, Map<String, dynamic> dataMap) {
    if (errorCall != null) {
      errorCall!(requestId, message, dataMap);
    }
    this.onComplete();
  }

  @override
  void onSuccess(Map<String, dynamic> dataMap) {
    if (successCall != null) {
      successCall!(requestId, dataMap);
    }
    this.onComplete();
  }
}

class _InternalStringCall with OnResponseStringCall {
  final OnResponseStringSuccessCall? successCall;
  final OnResponseStringErrorCall? errorCall;
  final OnResponseCompleteCall? completeCall;
  final String requestId;
  final OnResponseInterceptor? responseInterceptor;

  _InternalStringCall(this.requestId, this.successCall, this.errorCall, this.completeCall, this.responseInterceptor);

  @override
  void onComplete() {
    if (completeCall != null) {
      completeCall!(requestId);
    }
  }

  @override
  void onError(String message, String responseData) {
    if (errorCall != null) {
      errorCall!(requestId, message, responseData);
    }
    if (completeCall != null) {
      completeCall!(requestId);
    }
  }

  void _parsingData(String responseData) {
    var resultMap = JsonUtils.fromJson(responseData);
    String type = resultMap["type"];
    if (type.isEmptyString || type == "error") {
      this.onError("", "");
    } else if (type == "success") {
      String response = resultMap["data"];
      responseInterceptor?.onResponseString(response);
      if (successCall != null) {
        successCall!(requestId, response);
      }
    }
    this.onComplete();
  }

  @override
  void onSuccess(String responseData) {
    _parsingData(responseData);
  }
}

class _InternalResponseStringCall with OnResponseStringCall {
  final OnResponseCall? responseCall;
  final OnRequestInterceptor? interceptor;
  final OnResponseInterceptor? responseInterceptor;

  _InternalResponseStringCall(this.responseCall, this.interceptor, this.responseInterceptor);

  @override
  void onComplete() {
    responseCall?.onComplete();
  }

  void _parsingData(String responseData) {
    var resultMap = JsonUtils.fromJson(responseData);
    String type = resultMap["type"];
    if (type.isEmptyString || type == "error") {
      responseCall?.onError("", {});
    } else if (type == "success") {
      String response = resultMap["data"];
      var map = JsonUtils.fromJson(response);
      //数据拦截
      responseInterceptor?.onResponseMap(map);
      //回调api请求
      var code = map[interceptor?.codeKey() ?? "code"];
      if (code is int) {
        if (isSuccess(code)) {
          responseCall?.onSuccess(map);
        } else {
          var message = map[interceptor?.msgKey() ?? "msg"];
          responseCall?.onError(message, map);
        }
      } else {
        responseCall?.onSuccess(map);
      }
    }
    responseCall?.onComplete();
  }

  bool isSuccess(int code) {
    var codes = interceptor?.successCode() ?? [0, 200];
    return codes.contains(code);
  }

  @override
  void onError(String message, String responseData) {
    responseCall?.onError(message, {});
    responseCall?.onComplete();
  }

  @override
  void onSuccess(String responseData) {
    _parsingData(responseData);
  }
}
