import 'dart:async';

import 'package:eyflutter_core/dio/beans/retrofit.dart';
import 'package:eyflutter_core/dio/dio_manager.dart';
import 'package:eyflutter_core/dio/enums/method.dart';
import 'package:eyflutter_core/ey_flutter.dart';
import 'package:eyflutter_core/kit/parts/config_manager.dart';
import 'package:eyflutter_core/kit/utils/json_utils.dart';
import 'package:eyflutter_core/kit/utils/map/map_extension.dart';
import 'package:eyflutter_core/kit/utils/set/list_extention.dart';
import 'package:eyflutter_core/kit/utils/string/json_extension.dart';
import 'package:eyflutter_core/kit/utils/string/string_extension.dart';
import 'package:eyflutter_core/lang/beans/lang_config_entry.dart';
import 'package:eyflutter_core/lang/beans/lang_entry.dart';
import 'package:eyflutter_core/lang/event/on_lang_change_state.dart';
import 'package:eyflutter_core/lang/event/on_lang_config.dart';
import 'package:eyflutter_core/storage/mmkv_utils.dart';
import 'package:flutter/material.dart';

class LangManager {
  factory LangManager() => _getInstance();

  static LangManager get instance => _getInstance();
  static LangManager? _instance;

  static LangManager _getInstance() {
    return _instance ??= new LangManager._internal();
  }

  /// lang type config key
  String get _langTypeConfigKey => "2d94b1ab596a34e7";

  String get _versionCacheKey => "3af4ea6affa51e36";

  //当前语言
  Locale? _currentLocale;
  Map<String, dynamic> _textMap = {};
  Map<String, String> _configTextMap = {};
  OnLangConfig? _langConfig;
  Map<String, LangEntry> _supportLangMap = {};
  bool _isUpdateSupportLang = false;
  LangEntry? _defaultLang;
  LangEntry? _tempDefaultLang;
  Map<int, OnLangChangeState> _langStateDispatchs = {};
  List<Locale> _supportLocales = [];
  LangEntry? _currentLang;
  List<LangConfigEntry>? _configEntries;
  LangConfigEntry? _cacheConfigEntry;
  String? _langData;

  LangManager._internal() {
    _defaultLang = LangEntry(langCode: "zh", countryCode: "CN");
    _reset();
  }

  void addChangeState(int hashCode, OnLangChangeState langState) {
    _langStateDispatchs[hashCode] = langState;
  }

  void removeChangeState(int hashCode) {
    _langStateDispatchs.remove(hashCode);
  }

  /// region 切换本地化语言
  void notifyAll(Locale? locale) async {
    if (locale == null) {
      return;
    }
    _reset();
    await getLangData();
    _langStateDispatchs.forEach((key, value) {
      value.dispatchLangState(locale);
    });
  }

  void _reset() {
    _textMap.clear();
    _currentLocale = null;
    _currentLang = null;
    _cacheConfigEntry = null;
    _langData = "";
  }

  /// endregion

  /// region 注册配置项

  List<LangConfigEntry>? _getLangEntries() {
    if (_configEntries == null) {
      var config = langConfig();
      _configEntries = config?.langEntries() ?? [];
    }
    return _configEntries;
  }

  /// [config] OnLangConfig实现类
  void registerConfig<T extends OnLangConfig>(T? config) {
    if (config == null) {
      return;
    }
    ConfigManager.instance.addConfig(_langTypeConfigKey, config);
    var langEntries = _getLangEntries();
    if (langEntries?.length == _supportLangMap.length) {
      _isUpdateSupportLang = false;
    } else {
      _isUpdateSupportLang = true;
    }
  }

  /// endregion

  /// region 获取语言配置对象
  OnLangConfig? langConfig() {
    if (_langConfig == null) {
      var config = ConfigManager.instance.getConfig(_langTypeConfigKey);
      if (config is OnLangConfig) {
        _langConfig = config;
      }
    }
    return _langConfig;
  }

  /// endregion

  /// region 获取支持的语言
  bool _checkSupportLang() {
    if (_isUpdateSupportLang || _supportLangMap.isEmptyMap()) {
      _isUpdateSupportLang = true;
    }
    return _isUpdateSupportLang;
  }

  Map<String, LangEntry> _supportLang() {
    if (!_checkSupportLang()) {
      _isUpdateSupportLang = false;
      return _supportLangMap;
    }
    var langEntries = _getLangEntries();
    if (langEntries?.isEmptyList ?? true) {
      _isUpdateSupportLang = true;
      return {};
    }
    _supportLangMap.clear();
    langEntries?.forEach((element) {
      _supportLangMap[element.langCode] = LangEntry(langCode: element.langCode, countryCode: element.countryCode);
    });
    _isUpdateSupportLang = false;
    return _supportLangMap;
  }

  Map<String, LangEntry> get supportLang => _supportLang();

  /// endregion

  /// region 获取默认语言

  LangEntry? _getDefaultLang() {
    if (_tempDefaultLang != null) {
      return _tempDefaultLang;
    }
    var langEntries = _getLangEntries();
    if (langEntries?.isEmptyList ?? true) {
      return _defaultLang;
    }
    langEntries?.forEach((element) {
      if (element.isDefault) {
        _tempDefaultLang = LangEntry(langCode: element.langCode, countryCode: element.countryCode);
        return;
      }
    });
    return _tempDefaultLang;
  }

  LangEntry? get defaultLang => _getDefaultLang();

  /// endregion

  /// region 获取当前被使用的语言

  Locale? _getCurrentLocale(OnLangConfig? config, Locale locale) {
    if ((locale.languageCode.isEmptyString)) {
      var defaultLang = _getDefaultLang();
      _currentLocale = Locale(defaultLang?.langCode ?? "", defaultLang?.countryCode ?? "");
    } else if (_currentLocale?.languageCode != locale.languageCode) {
      var langEntries = _getLangEntries();
      langEntries?.forEach((element) {
        if (element.langCode == locale.languageCode) {
          _currentLocale = locale;
          return;
        }
      });
      if (_currentLocale == null) {
        var defaultLang = _getDefaultLang();
        _currentLocale = Locale(defaultLang?.langCode ?? "", defaultLang?.countryCode ?? "");
      }
    }
    return _currentLocale;
  }

  Locale? currentLocale(Locale locale) {
    return _getCurrentLocale(langConfig(), locale);
  }

  LangEntry? _getCurrentLang() {
    if (_currentLang == null) {
      if (_currentLocale != null) {
        _currentLang =
            LangEntry(langCode: _currentLocale?.languageCode ?? "", countryCode: _currentLocale?.countryCode ?? "");
      } else {
        var lang = defaultLang;
        _currentLang = LangEntry(langCode: lang?.langCode ?? "", countryCode: lang?.countryCode ?? "");
      }
    }
    return _currentLang;
  }

  LangEntry? get currentLang => _getCurrentLang();

  /// endregion

  /// region 绑定本地语言
  void bindCurrentLocale(Locale locale) async {
    await getLangData();
    var config = langConfig();
    _getCurrentLocale(config, locale);
  }

  /// endregion

  /// region 本地化语言
  /// 获取支持本地化语言
  List<Locale> _getSupportLocales() {
    if (_supportLocales.isNotEmptyList) {
      return _supportLocales;
    }
    var entries = supportLang.entries;
    entries.forEach((element) {
      var entry = element.value;
      _supportLocales.add(Locale(entry.langCode, entry.countryCode));
    });
    return _supportLocales;
  }

  List<Locale> get supportedLocale => _getSupportLocales();

  /// 判断本地语言是否支持
  /// [deviceLocale] 当前设备语言
  /// [supportedLocals] 设置的语言
  bool isSupported(Locale deviceLocale) {
    var locales = supportedLocale;
    if (locales.isEmptyList) {
      return false;
    }
    bool flag = false;
    locales.forEach((element) {
      if (element.languageCode == deviceLocale.languageCode) {
        flag = true;
        return;
      }
    });
    return flag;
  }

  /// endregion

  /// 获取语言数据
  Future<String> getLangData() async {
    if (_langData?.isNotEmptyString ?? false) {
      return _langData ?? "";
    }
    var configLang = _getCacheConfigLang();
    if (configLang?.unique.isEmptyString ?? true) {
      return "";
    }
    var appKey = EyFlutter.instance.appConfig?.appKey ?? "";
    var data = await MmkvUtils.instance.getString("${appKey}_${configLang?.unique}");
    _langData = Uri.decodeFull(data ?? "");
    return Future.value(_langData);
  }

  /// region 获取对应语言内容
  LangConfigEntry? _getCacheConfigLang() {
    if (_cacheConfigEntry == null) {
      var lang = currentLang;
      var entries = _getLangEntries();
      if (entries.isNotEmptyList) {
        _cacheConfigEntry = entries?.firstWhere((element) => element.langCode == lang?.langCode);
      }
    }
    return _cacheConfigEntry;
  }

  /// region 同步语言

  Future<Map<String, dynamic>> _getLangFromApi(String appKey) async {
    if (appKey.isEmptyString) {
      return {};
    }
    //获取请求url
    var config = langConfig();
    var url = await config?.langApiUrl();
    if (url.isEmptyString) {
      return {};
    }
    //请求文案配置
    Retrofit retrofit = Retrofit(api: url!, method: Method.GET);
    var dataMap = await DioManager.instance.asyncRequest(retrofit: retrofit, params: {"appKey": appKey});
    if (dataMap.containsKey("data")) {
      var map = dataMap.getValue<Map<String, dynamic>>("data", {});
      if (map.isEmptyMap()) {
        return Future.value({});
      } else {
        return Future.value(map);
      }
    } else {
      return Future.value({});
    }
  }

  /// 获取语言资源包
  Future<String> get langPackage async {
    var langData = await getLangData();
    var appKey = EyFlutter.instance.appConfig?.appKey ?? "";
    if (langData.isNotEmptyString) {
      Map<String, dynamic> _map = JsonUtils.fromJson(langData);
      _cacheUnitLang(appKey, _map);
      return langData;
    }
    Map<String, dynamic> langMap = await _getLangFromApi(appKey);
    _cacheUnitLang(appKey, langMap);
    var langJson = JsonUtils.toJson(langMap);
    _cacheLangData(appKey, langJson);
    return Future.value(langJson);
  }

  void _cacheLangData(String appKey, String langJson) {
    this._langData = langJson;
    var configLang = _getCacheConfigLang();
    if (configLang?.unique.isEmptyString ?? true) {
      return;
    }
    var encodeFull = Uri.encodeFull(langJson);
    MmkvUtils.instance.putString("${appKey}_${configLang!.unique}", value: encodeFull);
  }

  void _cacheUnitLang(String appKey, Map<String, dynamic> map) {
    map.forEach((key, value) {
      var cacheData = JsonUtils.toJson(value);
      var encodeFull = Uri.encodeFull(cacheData);
      MmkvUtils.instance.putString("${appKey}_$key", value: encodeFull);
    });
  }

  /// 同步文案数据
  Future<void> syncTextsData() async {
    var appKey = EyFlutter.instance.appConfig?.appKey ?? "";
    var config = langConfig();
    int version = (await config?.langVersion()) ?? 0;
    var isUpdateTexts = await _isUpdateTexts(appKey, version);
    if (isUpdateTexts) {
      Map<String, dynamic> langMap = await _getLangFromApi(appKey);
      _cacheUnitLang(appKey, langMap);
      var langJson = JsonUtils.toJson(langMap);
      _cacheLangData(appKey, langJson);
      MmkvUtils.instance.putInt("${appKey}_${_versionCacheKey}", value: version, part: MMkvPart.state);
    }
  }

  Future<bool> _isUpdateTexts(String appKey, int version) async {
    int rawVersion = (await MmkvUtils.instance.getInt("${appKey}_${_versionCacheKey}", part: MMkvPart.state)) ?? 0;
    if (rawVersion < version) {
      return Future.value(true);
    }
    var configLang = _getCacheConfigLang();
    if (configLang?.unique.isEmptyString ?? true) {
      return Future.value(true);
    }
    var cacheLang = await MmkvUtils.instance.getString("${appKey}_${configLang?.unique}");
    if (cacheLang.isEmptyString) {
      return Future.value(true);
    }
    _langData = Uri.decodeFull(cacheLang ?? "");
    if (_langData.isEmptyJson) {
      return Future.value(true);
    }
    return Future.value(false);
  }

  /// endregion

  /// [key] 语言关键信息key
  String value(String key) {
    if (key.isEmptyString) {
      return "";
    }
    var text = _textMap.getValue<String>(key);
    if (text.isNotEmptyString) {
      return text ?? "";
    }
    text = _getLangkeyTexts(key);
    if (text.isNotEmptyString) {
      return text;
    }
    var data = _getTextFromData(key);
    return data;
  }

  String _getTextFromData(String key) {
    if (_langData?.isEmptyJson ?? true) {
      langPackage;
      return "";
    }
    Map<String, dynamic> map = JsonUtils.fromJson(_langData ?? "");
    if (map.isEmptyMap()) {
      langPackage;
      return "";
    }
    String _text = "";
    var langCode = currentLang?.langCode;
    map.forEach((_key, unitConfig) {
      if (unitConfig is Map<String, dynamic>) {
        var result = unitConfig[langCode];
        if (result == null) {
          _textMap[_key] = "";
        } else {
          _textMap[_key] = "$result";
        }
        if (_key == key) {
          _text = _textMap[_key];
        }
      }
    });
    return _text;
  }

  String _getLangkeyTexts(String key) {
    var configJson = _configTextMap[key];
    var langCode = currentLang?.langCode;
    if (configJson.isNotEmptyString) {
      Map<String, dynamic> map = JsonUtils.fromJson(configJson!);
      var text = map[langCode];
      if (text is String) {
        _textMap[key] = text;
        return text;
      }
    }
    _getConfigCache(key);
    return "";
  }

  void _getConfigCache(String key) {
    var stagingClass = _UnitConfigStagingClass(key, (String key, String value) {
      if (value.isEmptyString) {
        langPackage;
      } else {
        _configTextMap[key] = value;
      }
    });
    stagingClass.execute();
  }

  /// endregion
}

typedef _OnStagingCall = void Function(String key, String value);

class _UnitConfigStagingClass {
  final String key;
  final _OnStagingCall stagingCall;

  _UnitConfigStagingClass(this.key, this.stagingCall);

  void execute() {
    var appKey = EyFlutter.instance.appConfig?.appKey ?? "";
    MmkvUtils.instance.getString("${appKey}_${key}").then((value) {
      stagingCall(key, Uri.decodeFull(value ?? ""));
    });
  }
}
